'use strict';
var path = require('path');
var util = require('util');
var angularUtils = require('../util.js');
var spawn = require('child_process').spawn;
var yeoman = require('yeoman-generator');


var Generator = module.exports = function Generator(args, options) {
  yeoman.generators.Base.apply(this, arguments);
  this.argument('appname', { type: String, required: false });
  this.appname = this.appname || path.basename(process.cwd());
  this.appname = this._.camelize(this._.slugify(this._.humanize(this.appname)));

  this.option('app-suffix', {
    desc: 'Allow a custom suffix to be added to the module name',
    type: String,
    required: 'false'
  });
  this.scriptAppName = this.appname + angularUtils.appName(this);

  args = ['main'];

  if (typeof this.env.options.appPath === 'undefined') {
    try {
      this.env.options.appPath = require(path.join(process.cwd(), 'bower.json')).appPath;
    } catch (e) {}
    this.env.options.appPath = this.env.options.appPath || 'app';
  }

  this.appPath = this.env.options.appPath;

  if (typeof this.env.options.coffee === 'undefined') {
    this.option('coffee', {
      desc: 'Generate CoffeeScript instead of JavaScript'
    });

    // attempt to detect if user is using CS or not
    // if cml arg provided, use that; else look for the existence of cs
    if (!this.options.coffee &&
      this.expandFiles(path.join(this.appPath, '/scripts/**/*.coffee'), {}).length > 0) {
      this.options.coffee = true;
    }

    this.env.options.coffee = this.options.coffee;
  }

  if (typeof this.env.options.minsafe === 'undefined') {
    this.option('minsafe', {
      desc: 'Generate AngularJS minification safe code'
    });
    this.env.options.minsafe = this.options.minsafe;
    args.push('--minsafe');
  }

  this.hookFor('angular-wsi:common', {
    args: args
  });

  this.hookFor('angular-wsi:main', {
    args: args
  });

  this.hookFor('angular-wsi:controller', {
    args: args
  });

  this.on('end', function () {
    this.installDependencies({ skipInstall: this.options['skip-install'] });

    var enabledComponents = [];

    if (this.resourceModule) {
      enabledComponents.push('angular-resource/angular-resource.js');
    }

    if (this.cookiesModule) {
      enabledComponents.push('angular-cookies/angular-cookies.js');
    }

    if (this.sanitizeModule) {
      enabledComponents.push('angular-sanitize/angular-sanitize.js');
    }

    if (this.routeModule) {
      enabledComponents.push('angular-route/angular-route.js');
    }

    if (this.bootstrapModule) {
      enabledComponents.push('angular-bootstrap/ui-bootstrap.js');
    }

    this.invoke('karma:app', {
      options: {
        coffee: this.options.coffee,
        travis: true,
        'skip-install': this.options['skip-install'],
        components: [
          'angular/angular.js',
          'angular-mocks/angular-mocks.js'
        ].concat(enabledComponents)
      }
    });
  });

  this.pkg = JSON.parse(this.readFileAsString(path.join(__dirname, '../package.json')));
};

util.inherits(Generator, yeoman.generators.Base);

Generator.prototype.askForBootstrap = function askForBootstrap() {
  var cb = this.async();

  this.prompt([{
    type: 'confirm',
    name: 'bootstrap',
    message: 'Would you like to include Twitter Bootstrap (WSI Version)?',
    default: true
  }/*, {
    type: 'confirm',
    name: 'compassBootstrap',
    message: 'Would you like to use the SCSS version of Twitter Bootstrap with the Compass CSS Authoring Framework?',
    default: true,
    when: function (props) {
      return props.bootstrap;
    }
  }*/], function (props) {
    this.bootstrap = props.bootstrap;
   // this.compassBootstrap = props.compassBootstrap;

    cb();
  }.bind(this));
};

Generator.prototype.askForModules = function askForModules() {
  var cb = this.async();

  var prompts = [{
    type: 'checkbox',
    name: 'modules',
    message: 'Which modules would you like to include?',
    choices: [{
      value: 'resourceModule',
      name: 'angular-resource.js',
      checked: true
    }, {
      value: 'cookiesModule',
      name: 'angular-cookies.js',
      checked: true
    }, {
      value: 'sanitizeModule',
      name: 'angular-sanitize.js',
      checked: true
    }, {
      value: 'routeModule',
      name: 'angular-route.js',
      checked: true
    }, {
      value: 'ui-bt-module',
      name: 'ui-bootstrap.js',
      checked: true
    }]
  }];

  this.prompt(prompts, function (props) {
    var hasMod = function (mod) { return props.modules.indexOf(mod) !== -1; };
    this.resourceModule = hasMod('resourceModule');
    this.cookiesModule = hasMod('cookiesModule');
    this.sanitizeModule = hasMod('sanitizeModule');
    this.routeModule = hasMod('routeModule');
    this.bootstrapModule = hasMod('ui-bt-module');

    var angMods = [];

    if (this.cookiesModule) {
      angMods.push("'ngCookies'");
    }

    if (this.resourceModule) {
      angMods.push("'ngResource'");
    }
    if (this.sanitizeModule) {
      angMods.push("'ngSanitize'");
    }
    if (this.routeModule) {
      angMods.push("'ngRoute'");
    }

    if (this.bootstrapModule) {
      angMods.push("'ui.bootstrap'");
    }

    if (angMods.length) {
      this.env.options.angularDeps = "\n  " + angMods.join(",\n  ") +"\n";
    }

    cb();
  }.bind(this));
};

Generator.prototype.readIndex = function readIndex() {
  this.indexFile = this.engine(this.read('../../templates/common/index.html'), this);
};

// Waiting a more flexible solution for #138
Generator.prototype.bootstrapFiles = function bootstrapFiles() {
  //var sass = this.compassBootstrap;
  var files = [];
  var source = 'bower_components/bootstrap-wsi/';

  if (this.bootstrap) {
    files.push('dist/css/bootstrap.css');
    this.copy('fonts/glyphicons-halflings-regular.eot', 'app/fonts/glyphicons-halflings-regular.eot');
    this.copy('fonts/glyphicons-halflings-regular.ttf', 'app/fonts/glyphicons-halflings-regular.ttf');
    this.copy('fonts/glyphicons-halflings-regular.svg', 'app/fonts/glyphicons-halflings-regular.svg');
    this.copy('fonts/glyphicons-halflings-regular.woff', 'app/fonts/glyphicons-halflings-regular.woff');

    //copy ui-bootstrap templates
    this.copy('ui-bootstrap-templates/accordion/accordion-group.html', 'app/template/accordion/accordion-group.html');
    this.copy('ui-bootstrap-templates/accordion/accordion.html', 'app/template/accordion/accordion.html');
    this.copy('ui-bootstrap-templates/alert/alert.html', 'app/template/alert/alert.html');
    this.copy('ui-bootstrap-templates/carousel/carousel.html', 'app/template/carousel/carousel.html');
    this.copy('ui-bootstrap-templates/carousel/slide.html', 'app/template/carousel/slide.html');
    this.copy('ui-bootstrap-templates/datepicker/datepicker.html', 'app/template/datepicker/datepicker.html');
    this.copy('ui-bootstrap-templates/datepicker/popup.html', 'app/template/datepicker/popup.html');
    this.copy('ui-bootstrap-templates/modal/backdrop.html', 'app/template/modal/backdrop.html');
    this.copy('ui-bootstrap-templates/modal/window.html', 'app/template/modal/window.html');
    this.copy('ui-bootstrap-templates/pagination/pagination.html', 'app/template/pagination/pagination.html');
    this.copy('ui-bootstrap-templates/pagination/pager.html', 'app/template/pagination/pager.html');
    this.copy('ui-bootstrap-templates/popover/popover.html', 'app/template/popover/popover.html');
    this.copy('ui-bootstrap-templates/progressbar/progress.html', 'app/template/progressbar/progress.html');
    this.copy('ui-bootstrap-templates/progressbar/bar.html', 'app/template/progressbar/bar.html');
    this.copy('ui-bootstrap-templates/rating/rating.html', 'app/template/rating/rating.html');
    this.copy('ui-bootstrap-templates/tabs/tab.html', 'app/template/tabs/tab.html');
    this.copy('ui-bootstrap-templates/tabs/tabset-titles.html', 'app/template/tabs/tabset-titles.html');
    this.copy('ui-bootstrap-templates/tabs/tabset.html', 'app/template/tabs/tabset.html');
    this.copy('ui-bootstrap-templates/timepicker/timepicker.html', 'app/template/timepicker/timepicker.html');
    this.copy('ui-bootstrap-templates/tooltip/tooltip-popup.html', 'app/template/tooltip/tooltip-popup.html');
    this.copy('ui-bootstrap-templates/tooltip/tooltip-html-unsafe-popup.html', 'app/template/tooltip/tooltip-html-unsafe-popup.html');
    this.copy('ui-bootstrap-templates/typeahead/typeahead-match.html', 'app/template/typeahead/typeahead-match.html');
    this.copy('ui-bootstrap-templates/typeahead/typeahead-popup.html', 'app/template/typeahead/typeahead-popup.html');
  }

  //files.push('main.' + (sass ? 's' : '') + 'css');
  files.push('examples/worldskills-application/application.css');

  //files.forEach(function (file) {
    //this.copy(source + file, 'app/styles/' + file);
  //}.bind(this));

  this.indexFile = this.appendFiles({
    html: this.indexFile,
    fileType: 'css',
    optimizedPath: 'styles/main.css',
    sourceFileList: files.map(function (file) {
      return source + file.replace('.scss', '.css');
    }),
    searchPath: ['.tmp', 'app']
  });
};

Generator.prototype.bootstrapJS = function bootstrapJS() {
  if (!this.bootstrap) {
    return;  // Skip if disabled.
  }

  // Wire Twitter Bootstrap plugins
  this.indexFile = this.appendScripts(this.indexFile, 'scripts/plugins.js', [
    'bower_components/angular-bootstrap/ui-bootstrap.js'
    //'bower_components/bootstrap-wsi/js/affix.js',
    //'bower_components/bootstrap-wsi/js/affix.js',
    //'bower_components/bootstrap-wsi/js/alert.js',
    //'bower_components/bootstrap-wsi/js/button.js',
    //'bower_components/bootstrap-wsi/js/carousel.js',
    //'bower_components/bootstrap-wsi/js/transition.js',
    //'bower_components/bootstrap-wsi/js/collapse.js',
    //'bower_components/bootstrap-wsi/js/dropdown.js',
    //'bower_components/bootstrap-wsi/js/modal.js',
    //'bower_components/bootstrap-wsi/js/scrollspy.js',
    //'bower_components/bootstrap-wsi/js/tab.js',
    //'bower_components/bootstrap-wsi/js/tooltip.js',
    //'bower_components/bootstrap-wsi/js/popover.js'
  ]);
};

Generator.prototype.extraModules = function extraModules() {
  var modules = [];
  if (this.resourceModule) {
    modules.push('bower_components/angular-resource/angular-resource.js');
  }

  if (this.cookiesModule) {
    modules.push('bower_components/angular-cookies/angular-cookies.js');
  }

  if (this.sanitizeModule) {
    modules.push('bower_components/angular-sanitize/angular-sanitize.js');
  }

  if (this.routeModule) {
    modules.push('bower_components/angular-route/angular-route.js');
  }

  if (this.bootstrapModule) {
    modules.push('bower_components/angular-bootstrap/ui-bootstrap.js');
  }

  if (modules.length) {
    this.indexFile = this.appendScripts(this.indexFile, 'scripts/modules.js',
        modules);
  }
};

Generator.prototype.appJs = function appJs() {
  this.indexFile = this.appendFiles({
    html: this.indexFile,
    fileType: 'js',
    optimizedPath: 'scripts/scripts.js',
    sourceFileList: ['scripts/app.js', 'scripts/controllers/main.js'],
    searchPath: ['.tmp', 'app']
  });
};

Generator.prototype.createIndexHtml = function createIndexHtml() {
  this.write(path.join(this.appPath, 'index.html'), this.indexFile);
};

Generator.prototype.packageFiles = function () {
  this.coffee = this.env.options.coffee;
  this.template('../../templates/common/_bower.json', 'bower.json');
  this.template('../../templates/common/_package.json', 'package.json');
  this.template('../../templates/common/Gruntfile.js', 'Gruntfile.js');
};

Generator.prototype.imageFiles = function () {
  this.sourceRoot(path.join(__dirname, 'templates'));
  this.directory('images', 'app/images', true);
}
